#!/usr/bin/env python3

import os
import pytest
from rf_info import rf_info
from random import randint
from rf_info import cli
from hypothesis import given, assume
from hypothesis.strategies import integers

MAX = 999999999999


TEST_SERVICES = (
                ('462562500', 'General Mobile Radio Service (GMRS)'),
)

TEST_BROADCAST = (
                 ('89.900.000', 'FM Radio'),
                 ('155.500', 'Longwave AM Radio International'),
                 ('55.500.000', 'VHF Television'),
                 ('510.000.000', 'UHF Television'),
)

TEST_IEEE = (
            ('1.525.500.000', 'L'),
            ('17.155.000.000', 'Ku'),
            ('34.567.121.000', 'Ka'),
            ('105.157.100.000', 'W'),
)

TEST_BANDTYPE = (
                ('132.000.000', 'Air Band'),
                ('166.580.000', 'Marine VHF'),
                ('27.405.000', 'CB'),
                ('462.562.500', 'GMRS'),
)

TEST_DISPLAY = (
            ('132.158.000', '132.158.000'),
            ('1,000,000', '1.000.000'),
            (1000, '1.000'),
            (142.233, '142.233'),
            (3542.23, '354.223'),
            ('988,232.000', '988.232.000'),
            ('12,000.000', '12.000.000'),
            ('1', '1'),
            ('.012', '12'),
            (.001, '1'),
)

TEST_UNITS = (
            ('132.158.000', 'Hz', '132.158.000'),
            ('1,000,000', '', '1.000.000'),
            ('1000', 'HZ', '1.000'),
            ('142.233', 'khz', '142.233'),
            ('3542.23', 'mhz', '3.542.230.000'),
            ('988,232.000', 'HZ', '988.232.000'),
            ('12,000.000', '', '12.000.000'),
            ('1', 'ghz', '1.000.000.000'),
            ('.012', 'GHZ', '12.000.000'),
            ('.001', 'Mhz', '1.000'),
            ('10', 'gHZ', '10.000.000.000'),
            ('300', 'mhZ', '300.000.000'),
)


RAND_TESTS = 500


def test_out_of_range():
    with pytest.raises(Exception) as e:
        freq = MAX + 1
        assert rf_info.Frequency(freq)
    assert str(e.value) == "Frequency Out of Range"

    with pytest.raises(Exception) as e:
        freq = -234
        assert rf_info.Frequency(freq)
    assert str(e.value) == "Invalid Frequency Specified"


def test_tiny_random():
    template = '{0:20s} | {1:10s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'ITU', '=='))
    rand_tests = RAND_TESTS
    while rand_tests > 0:
        random = randint(1, 9999)
        result = rf_info.Frequency(random).__dict__
        ok = 'OK' if isinstance(result, dict) else 'XX'
        print(template.format(str(random), result['itu_abbr'], ok))
        assert isinstance(result, dict)
        rand_tests -= 1


def test_small_random():
    template = '{0:20s} | {1:10s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'ITU', '=='))
    rand_tests = RAND_TESTS
    while rand_tests > 0:
        random = randint(10000, 999999)
        result = rf_info.Frequency(random).__dict__
        ok = 'OK' if isinstance(result, dict) else 'XX'
        print(template.format(str(random), result['itu_abbr'], ok))
        assert isinstance(result, dict)
        rand_tests -= 1


def test_medium_random():
    template = '{0:20s} | {1:10s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'ITU', '=='))
    rand_tests = RAND_TESTS
    while rand_tests > 0:
        random = randint(1000000, MAX)
        result = rf_info.Frequency(random).__dict__
        ok = 'OK' if isinstance(result, dict) else 'XX'
        print(template.format(str(random), result['itu_abbr'], ok))
        assert isinstance(result, dict)
        rand_tests -= 1


def test_large_random():
    template = '{0:20s} | {1:10s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'ITU', '=='))
    rand_tests = RAND_TESTS
    while rand_tests > 0:
        random = randint(1000000000, MAX)
        result = rf_info.Frequency(random).__dict__
        ok = 'OK' if isinstance(result, dict) else 'XX'
        print(template.format(str(random), result['itu_abbr'], ok))
        assert isinstance(result, dict)
        rand_tests -= 1


def test_types():
    template = '{0:20s} | {1:20s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'TYPE', '=='))

    result = rf_info.Frequency('1')
    assert isinstance(result.info(), dict)

    for (freq, expected) in TEST_DISPLAY:
        result = rf_info.Frequency(freq).__dict__
        ok = 'OK' if isinstance(result, dict) else 'XX'
        print(template.format(str(freq), f'{type(freq)}', ok))
        assert isinstance(result, dict)

    with pytest.raises(Exception) as e:
        freq = [1, 2, 3]
        print(template.format(str(freq), f'{type(freq)}', '--'))
        assert rf_info.Frequency(freq)
    assert str(e.value) == "Invalid Frequency Type"

    with pytest.raises(Exception) as e:
        freq = (1, 2, 3)
        print(template.format(str(freq), f'{type(freq)}', '--'))
        assert rf_info.Frequency(freq)
    assert str(e.value) == "Invalid Frequency Type"

    with pytest.raises(Exception) as e:
        freq = True
        print(template.format(str(freq), f'{type(freq)}', '--'))
        assert rf_info.Frequency(freq)
    assert str(e.value) == "Invalid Frequency Type"

    with pytest.raises(Exception) as e:
        freq = {'freq': 123}
        print(template.format(str(freq), f'{type(freq)}', '--'))
        assert rf_info.Frequency(freq)
    assert str(e.value) == "Invalid Frequency Type"


def test_invalids():
    template = '{0:20s} | {1:2s}'
    print(' ')
    print(template.format('FREQUENCY', '=='))
    with pytest.raises(Exception) as e:
        freq = '1i89'
        print(template.format(str(freq), '--'))
        assert rf_info.Frequency(freq)
    assert str(e.value) == "Invalid Frequency Specified"

    with pytest.raises(Exception) as e:
        freq = ['189', 'jk']
        print(template.format(str(freq), '--'))
        assert rf_info.Frequency(freq[0], freq[1])
    assert str(e.value) == "Invalid Unit Specified"



def test_dial():
    template = '{0:20s} | {1:20s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'DISPLAY', '=='))
    for (freq, expected) in TEST_DISPLAY:
        result = rf_info.Frequency(freq)
        ok = 'OK' if result.display == expected else 'XX'
        print(template.format(str(freq), str(result.display), ok))
        assert result.display == expected


def test_ieee():
    template = '{0:20s} | {1:20s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'IEEE', '=='))
    for (f, expected) in TEST_IEEE:
        result = rf_info.Frequency(f)
        ok = 'OK' if result.ieee_band == expected else 'XX'
        print(template.format(str(f), str(result.ieee_band), ok))
        assert result.ieee_band == expected


def test_band_use():
    template = '{0:20s} | {1:20s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'USE', '=='))
    for (f, expected) in TEST_BROADCAST:
        result = rf_info.Frequency(f)
        ok = 'OK' if expected in result.band_use else 'XX'
        print(template.format(str(f), str(result.band_use), ok))
        assert expected in result.band_use


def test_services():
    template = '{0:20s} | {1:20s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'SERVICE', '=='))
    for (f, expected) in TEST_SERVICES:
        result = rf_info.Frequency(f)
        ok = 'OK' if expected in result.band_use else 'XX'
        print(template.format(str(f), str(result.band_use), ok))
        assert expected in result.band_use


def test_units():
    template = '{0:20s} | {1:20s} | {3:20s} | {3:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'UNIT', 'RESULT', '=='))
    for (freq, unit, expected) in TEST_UNITS:
        result = rf_info.Frequency(freq, unit)
        ok = 'OK' if result.display == expected else 'XX'
        print(template.format(str(freq), unit, result.display, ok))
        assert result.display == expected


@given(integers(min_value=1, max_value=MAX))
def test_integers(a):
    result = rf_info.Frequency(a)
    assert isinstance(result.__dict__, dict)
    assert result.hz[1] == a
    assert result.khz[1] == a / 1000
    assert result.mhz[1] == a / 1000000
    assert result.ghz[1] == a / 1000000000



def test_cli():
    template = '{0:20s} | {1:20s} | {2:2s}'
    print(' ')
    print(template.format('FREQUENCY', 'suffix', '=='))
    for (freq, unit, expected) in TEST_UNITS:
        exit_status = os.system(f'rf-info {freq} {unit} > /dev/null')
        ok = 'OK' if exit_status == 0 else f'XX ({exit_status})'
        print(template.format(str(freq), unit, ok))
        assert exit_status == 0





from .rf_data import ITU, IEEE, NATO, WAVEGUIDE, BROADCAST, SERVICES, HAM


def remove_all_butfirst(s, substr):
    first_occurrence = s.index(substr) + len(substr)
    return s[:first_occurrence] + s[first_occurrence:].replace(substr, "")


def parse_freq(freq, unit):
    argfreq = freq.replace('.', '').replace(',', '').replace('_', '').replace(' ', '')
    if unit.lower() == 'khz':
        mindigits = 3
    elif unit.lower() == 'mhz':
        mindigits = 6
    elif unit.lower() == 'ghz':
        mindigits = 9
    elif unit.lower() == 'hz':
        if argfreq.isnumeric():
            return int(argfreq)
        else:
            raise ValueError('Invalid Frequency Specified')
    else:
        raise ValueError('Invalid Unit Specified')
    if '.' in freq:
        nfreq = remove_all_butfirst(freq, '.').split('.')
        while len(nfreq[1]) < mindigits:
            nfreq[1] = nfreq[1] + '0'
        return int(''.join(nfreq))
    else:
        for each in range(mindigits):
            argfreq = argfreq + '0'
        argfreq = str(int(argfreq))
        return int(argfreq)


class Frequency():

    def __init__(self, freq, unit='hz'):
        if unit == '':
            unit = 'hz'
        if (isinstance(freq, float) or isinstance(freq, str) or isinstance(freq, int)) and type(freq) != bool:
            intfreq = parse_freq(str(freq), unit)
        else:
            raise TypeError('Invalid Frequency Type')
        if intfreq < 1 or intfreq > 999999999999:
            raise ValueError(f'Frequency Out of Range')

        strfreq = str(intfreq)[::-1]
        strfreq = '.'.join(strfreq[i:i + 3] for i in range(0, len(strfreq), 3))
        self.display = strfreq[::-1]

        self.hz = ('{:,} hz'.format(int(intfreq)), (int(intfreq)))
        self.khz = ('{:,} Khz'.format(float(intfreq / 1000)), (float(intfreq / 1000)))
        self.mhz = ('{:,} Mhz'.format(float(intfreq / 1000000)), (float(intfreq / 1000000)))
        self.ghz = ('{:,} Ghz'.format(float(intfreq / 1000000000)), (float(intfreq / 1000000000)))

        itu = ITU[intfreq]
        ieee = IEEE[intfreq]
        meter = 300000000 / intfreq
        if meter >= 1:
            self.wavelength = '{:,}'.format(int(meter))
            self.wavelength = f'{self.wavelength}m'
        elif meter >= 0.01:
            sub = int(str(meter).split('.')[1])
            self.wavelength = f'{str(sub)[:2]}cm'
        elif meter < 0.01:
            sub = int(str(meter).split('.')[1]) * 1000
            self.wavelength = f'{str(sub)[:2]}mm'
        self.band_use = []

        if BROADCAST[intfreq] is not None and BROADCAST[intfreq]:
            self.band_use.append(BROADCAST[intfreq])

        if SERVICES[intfreq] is not None and SERVICES[intfreq]:
            self.band_use.append(SERVICES[intfreq])

        self.itu_band = itu[2]
        self.itu_abbr = itu[1]
        self.itu_num = itu[0]

        if ieee is not None:
            self.ieee_band = ieee[0]
            self.ieee_description = ieee[1]
        else:
            self.ieee_band = None
            self.ieee_description = None

        self.nato_band = NATO[intfreq]
        self.waveguide_band = WAVEGUIDE[intfreq]
        ham = HAM[intfreq]
        if ham is None:
            self.amateur_band = ((False, ))
        else:
            self.amateur_band = ((True, )) + ham

        if len(self.band_use) == 0:
            self.band_use = tuple()
        else:
            self.band_use = tuple(self.band_use)

    def info(self):
        return self.__dict__



from functools import reduce


class RangeKeyDict:
    def __init__(self, my_dict):
        assert not any(map(lambda x: not isinstance(x, tuple) or len(x) != 2 or x[0] > x[1], my_dict))

        def lte(bound):
            return lambda x: bound <= x

        def gt(bound):
            return lambda x: x < bound

        self._my_dict = {(lte(k[0]), gt(k[1])): v for k, v in my_dict.items()}

    def __getitem__(self, number):
        _my_dict = self._my_dict
        try:
            result = next((_my_dict[key] for key in _my_dict if list(reduce(lambda s, f: filter(f, s), key, [number]))))
        except StopIteration:
            return None
        else:
            return result



#!/usr/bin/env python3

import argparse
import sys

import rf_info


def main():
    # sys.tracebacklimit = 0  # Disable showing tracebacks
    parser = argparse.ArgumentParser()
    parser.add_argument('--version', '-v', action='version', version='%(prog)s {}'.format(rf_info.__version__))
    parser.add_argument('frequency', action='store', help='Radio Frequency to get information about')
    parser.add_argument('--raw', '-r', action='store_true', help='Includes raw output (for debugging)')
    parser.add_argument('unit', nargs='?', default='hz', help='hz, khz, Mhz, Ghz')
    args = parser.parse_args()

    result = rf_info.Frequency(str(args.frequency), str(args.unit).lower()).__dict__
    print(' ')
    for key, value in result.items():
        if key.lower() == 'band_use':
            if len(value) == 0:
                print('%s: %s' % (key.title(), 'Unknown'))
            else:
                newval = ''
                loop = 0
                for each in value:
                    if loop == 0:
                        loop += 1
                        newval = each
                    else:
                        newval = f'{newval}, {each}'
                print('%s: %s' % (key.title(), newval))

        elif key.lower() == 'hz' or key.lower() == 'khz' or key.lower() == 'mhz' or key.lower() == 'ghz':
            print('%s: %s' % (key.title(), value[0]))
        else:
            if value is not None:
                print('%s: %s' % (key.title(), value))
    print(' ')
    if args.raw:
        print('Raw:')
        for key, value in result.items():
            print(f'{key}: {value}')
        print(' ')
        print(result)
        print(' ')
    return 0


if __name__ == "__main__":
    sys.exit(main())



from .rf_info import Frequency

__author__ = """Ian Perry"""
__email__ = 'ianperry99@gmail.com'
__version__ = '0.4.3'
__license__ = 'MIT'
__progname__ = 'rf-info'
__description__ = "Radio Frequency Information"
